from django.forms.fields import ChoiceField
from django.forms.widgets import Select
from django.utils.translation import ugettext_lazy as _

from fobi.base import FormFieldPlugin, form_element_plugin_registry, get_theme
from fobi.helpers import get_select_field_choices

from . import UID
from .constants import SLIDER_DEFAULT_TOOLTIP, SLIDER_DEFAULT_HANDLE
from .forms import SliderInputForm
from .settings import INITIAL, MAX_VALUE, MIN_VALUE, STEP

__title__ = 'fobi.contrib.plugins.form_elements.fields.' \
            'slider_percentage.fobi_form_elements'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2014-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('SliderInputPlugin',)

theme = get_theme(request=None, as_instance=True)


class SliderInputPlugin(FormFieldPlugin):
    """Slider field plugin."""

    uid = UID
    name = _("Slider")
    group = _("Fields")
    form = SliderInputForm
    html_classes = ['slider']

    def get_form_field_instances(self, request=None):
        """Get form field instances."""
        initial = self.data.initial if self.data.initial else INITIAL
        max_value = self.data.max_value if self.data.max_value else MAX_VALUE
        min_value = self.data.min_value if self.data.min_value else MIN_VALUE
        step = self.data.step if self.data.step else STEP
        tooltip = self.data.tooltip \
            if self.data.tooltip \
            else SLIDER_DEFAULT_TOOLTIP
        handle = self.data.handle \
            if self.data.handle \
            else SLIDER_DEFAULT_HANDLE

        # custom_ticks = get_select_field_choices(self.data.custom_ticks) \
        #     if self.data.custom_ticks \
        #     else []

        _choices = range(min_value, max_value, step)
        choices = zip(_choices, _choices)

        # slider_html_class = "slider-no-background" \
        #     if self.data.disable_slider_background \
        #     else "slider"
        slider_html_class = "slider"

        widget_attrs = {
            'class': "{0} {1}".format(
                slider_html_class,
                theme.form_element_html_class
            ),
            'data-slider-min': min_value,
            'data-slider-max': max_value,
            'data-slider-step': step,
            'data-slider-tooltip': tooltip,
            'data-slider-handle': handle,
        }

        if self.data.enable_ticks:
            # if custom_ticks:
            #     pass
            # else:
                tick_label_start = self.data.tick_label_start \
                    if self.data.tick_label_start \
                    else min_value

                tick_label_end = self.data.tick_label_end \
                    if self.data.tick_label_end \
                    else max_value

                widget_attrs.update({
                    'data-slider-ticks': "[{0}, {1}]".format(
                        min_value, max_value
                    ),
                    'data-slider-ticks-labels': '["{0}", "{1}"]'.format(
                        tick_label_start, tick_label_end
                    ),
                })

        # I hate to do so, but it seems like a dirty workaround that works.
        if 'POST' == request.method:
            try:
                value = int(request.POST.get(self.data.name, initial))
            except (ValueError, TypeError) as err:
                value = initial

            if value < min_value or value > max_value:
                value = initial

            widget_attrs.update({
                'data-slider-value': value,
            })
        else:
            widget_attrs.update({
                'data-slider-value': initial,
            })

        kwargs = {
            'label': self.data.label,
            'help_text': self.data.help_text,
            'initial': initial,
            'required': self.data.required,
            'choices': choices,
            'widget': Select(attrs=widget_attrs),
        }

        return [(self.data.name, ChoiceField, kwargs)]


form_element_plugin_registry.register(SliderInputPlugin)



from fobi.integration.helpers import get_template_choices

from .settings import FORM_TEMPLATE_CHOICES, SUCCESS_PAGE_TEMPLATE_CHOICES

__title__ = 'fobi.contrib.apps.djangocms_integration.helpers'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2014-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'get_form_template_choices',
    'get_success_page_template_choices',
)


def get_form_template_choices():
    """Get the form template choices.

    It's possible to provide theme templates per theme or just per project.

    :return list:
    """
    return get_template_choices(
        'djangocms_integration',
        FORM_TEMPLATE_CHOICES,
        'form_template_choices'
    )


def get_success_page_template_choices():
    """Get success page template choices.

    :return list:
    """
    return get_template_choices(
        'djangocms_integration',
        SUCCESS_PAGE_TEMPLATE_CHOICES,
        'success_page_template_choices'
    )



from fobi.base import FormElementPluginWidget

from . import UID

__title__ = 'fobi.contrib.plugins.form_elements.test.dummy.widgets'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2014-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('BaseDummyPluginWidget',)


class BaseDummyPluginWidget(FormElementPluginWidget):
    """Base dummy form element plugin widget."""

    plugin_uid = UID



from six import text_type

import simplejson as json

from django.core.urlresolvers import reverse

from .base import BaseRegistry
from .discover import autodiscover

__title__ = 'fobi.form_importers'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2014-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'BaseFormImporter',
    'FormImporterPluginRegistry',
    'form_importer_plugin_registry',
    'ensure_autodiscover',
    'get_form_importer_plugin_uids',
    'get_form_impoter_plugin_urls',
)


class BaseFormImporter(object):
    """Base importer."""

    uid = None
    name = None
    description = None

    # field_type (MailChimp): uid (Fobi)
    fields_mapping = None

    # Django standard: remote
    field_properties_mapping = None

    # Position is stored in a model (field)
    field_type_prop_name = None
    position_prop_name = None
    wizard = None
    templates = None

    def __init__(self, form_entry_cls, form_element_entry_cls,
                 form_properties=None, form_data=None):
        """Constructor.

        :param django.contrib.auth.models.User user: User importing the form.
        :param dict form_properties: Properties of the form, that
            user provides (such as name, is_public, etc.)
        """
        assert self.uid
        assert self.name
        assert self.fields_mapping is not None
        assert self.field_properties_mapping is not None
        assert self.wizard is not None
        assert self.templates is not None

        assert form_entry_cls is not None
        assert form_element_entry_cls is not None

        for prop in ('name', 'label', 'help_text', 'initial', 'required'):
            assert prop in self.field_properties_mapping

        self.form_data = form_data
        self.form_properties = form_properties
        self.form_entry_cls = form_entry_cls
        self.form_element_entry_cls = form_element_entry_cls

    def get_form_data(self):
        """Get form data."""
        return self.form_data

    def extract_field_properties(self, field_data):
        """Extract field properties."""
        field_properties = {}
        for prop, val in self.field_properties_mapping.items():
            if val in field_data:
                field_properties[prop] = field_data[val]
        return field_properties

    def import_data(self, form_properties, form_data):
        """Import data."""
        self.form_properties = form_properties
        self.form_data = form_data

        assert 'name' in self.form_properties
        form_entry = self.form_entry_cls()
        for prop, val in self.form_properties.items():
            setattr(form_entry, prop, val)

        form_entry.save()

        data = self.get_form_data()
        for field_data in data:
            # Skip non-existing
            if field_data[self.field_type_prop_name] \
                    not in self.fields_mapping:
                continue

            form_element_entry = self.form_element_entry_cls()
            form_element_entry.form_entry = form_entry
            form_element_entry.plugin_uid = self.fields_mapping[
                field_data[self.field_type_prop_name]]

            # Assign form data
            form_element_entry.plugin_data = json.dumps(
                self.extract_field_properties(field_data)
            )

            # Assign position in form
            if self.position_prop_name in field_data:
                form_element_entry.position = field_data[
                    self.position_prop_name
                ]

            form_element_entry.save()

        return form_entry

    def get_template_names(self):
        """Get template names."""
        return {text_type(idx): tpl for idx, tpl in enumerate(self.templates)}

    def get_wizard(self, request, *args, **kwargs):
        """Get wizard."""
        template_names = self.get_template_names()

        class FormImporterWizard(self.wizard):
            """Constructing the importer class dynamically."""

            _form_importer = self

            def get_template_names(self):
                """Get template names."""
                return [template_names[self.steps.current]]

        wizard = FormImporterWizard.as_view()
        return wizard(request, *args, **kwargs)


class FormImporterPluginRegistry(BaseRegistry):
    """Form importer plugins registry."""

    type = BaseFormImporter


# Register form field plugins by calling form_field_plugin_registry.register()
form_importer_plugin_registry = FormImporterPluginRegistry()


def ensure_autodiscover():
    """Ensure that form importer plugins are auto-discovered."""
    if not (form_importer_plugin_registry._registry):
        autodiscover()


def get_form_importer_plugin_uids():
    """Get form importer plugin uids."""
    ensure_autodiscover()
    return list(form_importer_plugin_registry._registry.keys())


def get_form_impoter_plugin_urls():
    """Gets the form importer plugin URLs as a list of tuples."""
    urls = []
    ensure_autodiscover()
    for uid, plugin in form_importer_plugin_registry._registry.items():
        urls.append(
            (
                uid,
                plugin.name,
                reverse('fobi.form_importer',
                        kwargs={'form_importer_plugin_uid': uid})
            )
        )
    return urls



from django import forms
from django.core.validators import ip_address_validator_map
from django.utils.translation import ugettext_lazy as _

from fobi.base import BaseFormFieldPluginForm, get_theme
from fobi.settings import DEFAULT_MAX_LENGTH

__title__ = 'fobi.contrib.plugins.form_elements.fields.ip_address.forms'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2014-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('TextInputForm',)

theme = get_theme(request=None, as_instance=True)


class IPAddressInputForm(forms.Form, BaseFormFieldPluginForm):
    """Form for ``IPAddressInputPlugin``."""

    plugin_data_fields = [
        ("label", ""),
        ("name", ""),
        ("help_text", ""),
        ("initial", ""),
        ("protocol", ""),
        ("unpack_ipv4", False),
        ("max_length", "255"),
        ("required", False),
        ("placeholder", ""),
    ]

    label = forms.CharField(
        label=_("Label"),
        required=True,
        widget=forms.widgets.TextInput(
            attrs={'class': theme.form_element_html_class}
        )
    )
    name = forms.CharField(
        label=_("Name"),
        required=True,
        widget=forms.widgets.TextInput(
            attrs={'class': theme.form_element_html_class}
        )
    )
    help_text = forms.CharField(
        label=_("Help text"),
        required=False,
        widget=forms.widgets.Textarea(
            attrs={'class': theme.form_element_html_class}
        )
    )
    initial = forms.CharField(
        label=_("Initial"),
        required=False,
        widget=forms.widgets.TextInput(
            attrs={'class': theme.form_element_html_class}
        )
    )
    protocol = forms.ChoiceField(
        label=_("Protocol"),
        choices=[(pr, pr) for pr in ip_address_validator_map.keys()],
        required=True,
        widget=forms.widgets.Select(
            attrs={'class': theme.form_element_html_class}
        )
    )
    unpack_ipv4 = forms.BooleanField(
        label=_("Unpack IPV4"),
        required=False,
        widget=forms.widgets.CheckboxInput(
            attrs={'class': theme.form_element_checkbox_html_class}
        )
    )
    max_length = forms.IntegerField(
        label=_("Max length"),
        required=True,
        widget=forms.widgets.TextInput(
            attrs={'class': theme.form_element_html_class}
        ),
        initial=DEFAULT_MAX_LENGTH
    )
    required = forms.BooleanField(
        label=_("Required"),
        required=False,
        widget=forms.widgets.CheckboxInput(
            attrs={'class': theme.form_element_checkbox_html_class}
        )
    )
    placeholder = forms.CharField(
        label=_("Placeholder"),
        required=False,
        widget=forms.widgets.TextInput(
            attrs={'class': theme.form_element_html_class}
        )
    )



from .conf import get_setting

__title__ = 'fobi.contrib.plugins.form_elements.fields.' \
            'hidden_model_object.settings'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = 'Copyright (c) 2014-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('IGNORED_MODELS',)

IGNORED_MODELS = get_setting('IGNORED_MODELS')



from fobi.base import form_handler_plugin_widget_registry
from fobi.contrib.plugins.form_handlers.db_store.widgets import (
    BaseDbStorePluginWidget
)
from fobi.contrib.themes.foundation5 import UID

__title__ = 'fobi.contrib.themes.foundation5.widgets.form_handlers.' \
            'db_store_foundation5_widget.fobi_form_elements'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2014-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('DbStorePluginWidget',)


class DbStorePluginWidget(BaseDbStorePluginWidget):
    """DbStore plugin widget for Foundation 5."""

    theme_uid = UID
    view_entries_icon_class = 'fi-list'
    export_entries_icon_class = 'fi-page-export'
    view_saved_form_data_entries_template_name = \
        'db_store_foundation5_widget/view_saved_form_data_entries.html'


# Registering the widget
form_handler_plugin_widget_registry.register(DbStorePluginWidget)



from uuid import uuid4

from django.conf import settings
from django.template.loader import render_to_string
from django.utils.translation import ugettext_lazy as _

from nonefield.fields import NoneField

from fobi.base import FormElementPlugin, form_element_plugin_registry
from fobi.helpers import delete_file, clone_file

from . import UID
from .forms import ContentImageForm
from .helpers import get_crop_filter
from .settings import (
    FIT_METHOD_FIT_WIDTH, FIT_METHOD_FIT_HEIGHT, IMAGES_UPLOAD_DIR
)

__title__ = 'fobi.contrib.plugins.form_elements.content.content_image.' \
            'fobi_form_elements'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2014-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('ContentImagePlugin',)


class ContentImagePlugin(FormElementPlugin):
    """Content image plugin."""

    uid = UID
    name = _("Content image")
    group = _("Content")
    form = ContentImageForm

    def post_processor(self):
        """Post process data.

        Always the same.
        """
        self.data.name = "{0}_{1}".format(self.uid, uuid4())

    def delete_plugin_data(self):
        """Delete uploaded file."""
        delete_file(self.data.file)

    def clone_plugin_data(self, entry):
        """Clone plugin data.

        Clone plugin data, which means we make a copy of the original image.

        TODO: Perhaps rely more on data of ``form_element_entry``?
        """
        cloned_image = clone_file(
            IMAGES_UPLOAD_DIR, self.data.file, relative_path=True
        )
        return self.get_cloned_plugin_data(update={'file': cloned_image})

    def get_form_field_instances(self, request=None):
        """Get form field instances."""
        width, height = self.data.size.split('x')
        crop = get_crop_filter(self.data.fit_method)

        if FIT_METHOD_FIT_WIDTH == self.data.fit_method:
            thumb_size = (width, 0)
        elif FIT_METHOD_FIT_HEIGHT == self.data.fit_method:
            thumb_size = (0, height)
        else:
            thumb_size = (width, height)

        context = {
            'plugin': self,
            'MEDIA_URL': settings.MEDIA_URL,
            'crop': crop,
            'thumb_size': thumb_size
        }
        rendered_image = render_to_string('content_image/render.html', context)

        kwargs = {
            'initial': rendered_image,
            'required': False,
            'label': '',
        }

        return [(self.data.name, NoneField, kwargs)]


form_element_plugin_registry.register(ContentImagePlugin)



from django.forms.fields import NullBooleanField
from django.forms.widgets import NullBooleanSelect
from django.utils.translation import ugettext_lazy as _

from fobi.base import FormFieldPlugin, form_element_plugin_registry, get_theme

from . import UID
from .forms import NullBooleanSelectForm

__title__ = 'fobi.contrib.plugins.form_elements.fields.' \
            'null_boolean.fobi_form_elements'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2014-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('NullBooleanSelectPlugin',)

theme = get_theme(request=None, as_instance=True)


class NullBooleanSelectPlugin(FormFieldPlugin):
    """Null boolean select plugin."""

    uid = UID
    name = _("Null boolean")
    group = _("Fields")
    form = NullBooleanSelectForm

    def get_form_field_instances(self, request=None):
        """Get form field instances."""
        kwargs = {
            'label': self.data.label,
            'help_text': self.data.help_text,
            'initial': self.data.initial,
            'required': self.data.required,
            'widget': NullBooleanSelect(
                attrs={'class': theme.form_element_html_class}
            ),
        }

        return [(self.data.name, NullBooleanField, kwargs)]


form_element_plugin_registry.register(NullBooleanSelectPlugin)



from django.utils.translation import ugettext_lazy as _

__all__ = (
    'SLIDER_TOOLTIP_SHOW',
    'SLIDER_TOOLTIP_HIDE',
    'SLIDER_TOOLTIP_ALWAYS',
    'SLIDER_DEFAULT_TOOLTIP',
    'SLIDER_TOOLTIP_CHOICES',

    'SLIDER_HANDLE_ROUND',
    'SLIDER_HANDLE_SQUARE',
    'SLIDER_HANDLE_TRIANGLE',
    'SLIDER_HANDLE_CUSTOM',
    'SLIDER_HANDLE_CHOICES',
)

SLIDER_TOOLTIP_SHOW = 'show'
SLIDER_TOOLTIP_HIDE = 'hide'
SLIDER_TOOLTIP_ALWAYS = 'always'
SLIDER_DEFAULT_TOOLTIP = SLIDER_TOOLTIP_HIDE

SLIDER_TOOLTIP_CHOICES = (
    (SLIDER_TOOLTIP_SHOW, _("Show")),
    (SLIDER_TOOLTIP_HIDE, _("Hide")),
    (SLIDER_TOOLTIP_ALWAYS, _("Always")),
)

SLIDER_HANDLE_ROUND = 'round'
SLIDER_HANDLE_SQUARE = 'square'
SLIDER_HANDLE_TRIANGLE = 'triangle'
SLIDER_HANDLE_CUSTOM = 'custom'
SLIDER_DEFAULT_HANDLE = SLIDER_HANDLE_ROUND
SLIDER_HANDLE_CHOICES = (
    (SLIDER_HANDLE_ROUND, _("Round")),
    (SLIDER_HANDLE_SQUARE, _("Square")),
    (SLIDER_HANDLE_TRIANGLE, _("Triangle")),
    (SLIDER_HANDLE_CUSTOM, _("Custom")),
)



from django import forms
from django.utils.translation import ugettext_lazy as _

from fobi.base import BaseFormFieldPluginForm, get_theme
from fobi.helpers import validate_initial_for_multiple_choices
from fobi.widgets import NumberInput

__title__ = 'fobi.contrib.plugins.form_elements.fields.select_multiple.forms'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2014-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('SelectMultipleWithMaxInputForm',)

theme = get_theme(request=None, as_instance=True)


class SelectMultipleWithMaxInputForm(forms.Form, BaseFormFieldPluginForm):
    """Form for ``SelectMultipleWithMaxInputPlugin``."""

    plugin_data_fields = [
        ("label", ""),
        ("name", ""),
        ("choices", ""),
        ("help_text", ""),
        ("initial", ""),
        ("required", False),
        ("max_choices", "")
    ]

    label = forms.CharField(
        label=_("Label"),
        required=True,
        widget=forms.widgets.TextInput(
            attrs={'class': theme.form_element_html_class}
        )
    )
    name = forms.CharField(
        label=_("Name"),
        required=True,
        widget=forms.widgets.TextInput(
            attrs={'class': theme.form_element_html_class}
        )
    )
    choices = forms.CharField(
        label=_("Choices"),
        required=False,
        help_text=_("Enter single values/pairs per line. Example:<code><br/>"
                    "&nbsp;&nbsp;&nbsp;&nbsp;1<br/>"
                    "&nbsp;&nbsp;&nbsp;&nbsp;2<br/>"
                    "&nbsp;&nbsp;&nbsp;&nbsp;alpha, Alpha<br/>"
                    "&nbsp;&nbsp;&nbsp;&nbsp;beta, Beta<br/>"
                    "&nbsp;&nbsp;&nbsp;&nbsp;omega"
                    "</code><br/>"
                    "It finally transforms into the following HTML "
                    "code:<code><br/>"
                    '&nbsp;&nbsp;&nbsp;&nbsp;'
                    '&lt;select id="id_NAME_OF_THE_ELEMENT" '
                    'name="NAME_OF_THE_ELEMENT"&gt;<br/>'
                    '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;'
                    '&lt;option value="1"&gt;1&lt;/option&gt;<br/>'
                    '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;'
                    '&lt;option value="2"&gt;2&lt;/option&gt;<br/>'
                    '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;'
                    '&lt;option value="alpha"&gt;Alpha&lt;/option&gt;<br/>'
                    '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;'
                    '&lt;option value="beta"&gt;Beta&lt;/option&gt;<br/>'
                    '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;'
                    '&lt;option value="omega"&gt;omega&lt;/option&gt;<br/>'
                    '&nbsp;&nbsp;&nbsp;&nbsp;&lt;/select&gt;'
                    "</code>"),
        widget=forms.widgets.Textarea(
            attrs={'class': theme.form_element_html_class}
        )
    )
    help_text = forms.CharField(
        label=_("Help text"),
        required=False,
        widget=forms.widgets.Textarea(
            attrs={'class': theme.form_element_html_class}
        )
    )
    initial = forms.CharField(
        label=_("Initial"),
        required=False,
        widget=forms.widgets.TextInput(
            attrs={'class': theme.form_element_html_class}
        )
    )
    required = forms.BooleanField(
        label=_("Required"),
        required=False,
        widget=forms.widgets.CheckboxInput(
            attrs={'class': theme.form_element_checkbox_html_class}
        )
    )
    max_choices = forms.IntegerField(
        label=_("Max choices"),
        required=False,
        widget=NumberInput(
            attrs={'class': theme.form_element_html_class}
        )
    )

    def clean_initial(self):
        """Validating the initial value."""
        return validate_initial_for_multiple_choices(self,
                                                     'choices',
                                                     'initial')



import logging
import os

import requests

from django.conf import settings
from django.template.loader import render_to_string
from django.utils.translation import ugettext_lazy as _

from fobi.base import (
    FormHandlerPlugin,
    FormWizardHandlerPlugin,
    form_handler_plugin_registry,
    form_wizard_handler_plugin_registry
)
from fobi.helpers import (
    extract_file_path,
    get_form_element_entries_for_form_wizard_entry
)

from . import UID
from .forms import HTTPRepostForm

__title__ = 'fobi.contrib.plugins.form_handlers.http_repost.fobi_form_handlers'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2014-2015 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'HTTPRepostHandlerPlugin',
    'HTTPRepostWizardHandlerPlugin',
)

logger = logging.getLogger(__name__)

# *****************************************************************************
# **************************** Form handler ***********************************
# *****************************************************************************


class HTTPRepostHandlerPlugin(FormHandlerPlugin):
    """HTTP repost handler plugin.

    Makes a HTTP repost to a given endpoint. Should be executed before
    ``db_store`` plugin.
    """

    uid = UID
    name = _("HTTP Repost")
    form = HTTPRepostForm

    def run(self, form_entry, request, form, form_element_entries=None):
        """Run.

        :param fobi.models.FormEntry form_entry: Instance of
            ``fobi.models.FormEntry``.
        :param django.http.HttpRequest request:
        :param django.forms.Form form:
        :param iterable form_element_entries: Iterable of
            ``fobi.models.FormElementEntry`` objects.
        """
        files = self._prepare_files(request, form)
        try:
            response = requests.post(
                self.data.endpoint_url,
                data=request.POST.dict(),
                files=files,
                allow_redirects=True,
                timeout=5
            )
            return (True, response)
        except Exception as err:
            logger.debug(str(err))
            return (False, err)

    def _prepare_files(self, request, form):
        """Prepares the files for being attached to the mail message."""
        files = {}

        def process_path(file_path, imf):
            if file_path:
                # if file_path.startswith(settings.MEDIA_URL):
                #     file_path = file_path[1:]
                # file_path = settings.PROJECT_DIR('../{0}'.format(file_path))
                file_path = file_path.replace(
                    settings.MEDIA_URL,
                    os.path.join(settings.MEDIA_ROOT, '')
                )
                files[field_name] = (imf.name, open(file_path, 'rb'))

        for field_name, imf in request.FILES.items():
            try:
                file_path = form.cleaned_data.get(field_name, '')
                process_path(file_path, imf)
            except Exception as err:
                file_path = extract_file_path(imf.name)
                process_path(file_path, imf)

        return files

    def plugin_data_repr(self):
        """Human readable representation of plugin data.

        :return string:
        """
        context = {
            'endpoint_url': self.data.endpoint_url,
        }
        return render_to_string('http_repost/plugin_data_repr.html', context)


form_handler_plugin_registry.register(HTTPRepostHandlerPlugin)


# *****************************************************************************
# ************************ Form wizard handler ********************************
# *****************************************************************************


class HTTPRepostWizardHandlerPlugin(FormWizardHandlerPlugin):
    """HTTP repost wizard handler plugin.

    Makes a HTTP repost to a given endpoint. Should be executed before
    ``db_store`` plugin.
    """

    uid = UID
    name = _("HTTP Repost")
    form = HTTPRepostForm

    def run(self, form_wizard_entry, request, form_list, form_wizard,
            form_element_entries=None):
        """Run.

        :param fobi.models.FormWizardEntry form_wizard_entry: Instance
            of :class:`fobi.models.FormWizardEntry`.
        :param django.http.HttpRequest request:
        :param list form_list: List of :class:`django.forms.Form` instances.
        :param fobi.wizard.views.dynamic.DynamicWizardView form_wizard:
            Instance of :class:`fobi.wizard.views.dynamic.DynamicWizardView`.
        :param iterable form_element_entries: Iterable of
            ``fobi.models.FormElementEntry`` objects.
        """
        files = self._prepare_files(request, form_list)
        try:
            response = requests.post(
                self.data.endpoint_url,
                data=request.POST.dict(),
                files=files,
                allow_redirects=True,
                timeout=5
            )
            return (True, response)
        except Exception as err:
            logger.debug(str(err))
            return (False, err)

    def _prepare_files(self, request, form_list):
        """Prepares the files for being attached to the mail message."""
        files = {}

        def process_path(file_path, imf):
            if file_path:
                # if file_path.startswith(settings.MEDIA_URL):
                #     file_path = file_path[1:]
                # file_path = settings.PROJECT_DIR('../{0}'.format(file_path))
                file_path = file_path.replace(
                    settings.MEDIA_URL,
                    os.path.join(settings.MEDIA_ROOT, '')
                )
                files[field_name] = (imf.name, open(file_path, 'rb'))

        for form in form_list:
            for field_name, imf in request.FILES.items():
                try:
                    file_path = form.cleaned_data.get(field_name, '')
                    process_path(file_path, imf)
                except Exception as err:
                    file_path = extract_file_path(imf.name)
                    process_path(file_path, imf)

        return files

    def plugin_data_repr(self):
        """Human readable representation of plugin data.

        :return string:
        """
        context = {
            'endpoint_url': self.data.endpoint_url,
        }
        return render_to_string('http_repost/plugin_data_repr.html', context)


form_wizard_handler_plugin_registry.register(HTTPRepostWizardHandlerPlugin)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.9 on 2016-10-15 23:35
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('fobi', '0010_formwizardhandler'),
        ('fobi_contrib_plugins_form_handlers_db_store', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='SavedFormWizardDataEntry',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('form_data_headers', models.TextField(blank=True, null=True, verbose_name='Form data headers')),
                ('saved_data', models.TextField(blank=True, null=True, verbose_name='Plugin data')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Date created')),
                ('form_wizard_entry', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='fobi.FormWizardEntry', verbose_name='Form')),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'abstract': False,
                'db_table': 'db_store_savedformwizarddataentry',
                'verbose_name': 'Saved form wizard data entry',
                'verbose_name_plural': 'Saved form wizard data entries',
            },
        ),
    ]



from django import forms
from django.conf import settings
from django.template import Library, TemplateSyntaxError, Node
from django.utils.translation import ugettext_lazy as _

from nine.versions import DJANGO_GTE_1_7

from fobi.base import get_theme
from fobi.settings import DISPLAY_AUTH_LINK

if DJANGO_GTE_1_7:
    from django.forms.utils import ErrorDict
else:
    from django.forms.util import ErrorDict

__title__ = 'fobi.templatetags.fobi_tags'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2014-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'get_fobi_plugin',
    'get_fobi_form_handler_plugin_custom_actions',
    'get_fobi_form_wizard_handler_plugin_custom_actions',
    'get_form_field_type',
    'get_form_hidden_fields_errors',
    'has_edit_form_entry_permissions',
    'render_auth_link',
    'render_fobi_forms_list',
)

theme = get_theme(request=None, as_instance=True)

register = Library()

# *****************************************************************************
# *****************************************************************************
# *****************************************************************************
# **************************** General Fobi tags ******************************
# *****************************************************************************
# *****************************************************************************
# *****************************************************************************

# *****************************************************************************
# ******************************* Plugin specific *****************************
# *****************************************************************************


class GetFobiPluginNode(Node):
    """Node for ``get_fobi_plugin`` tag."""

    def __init__(self, entry, as_var=None):
        """Constructor."""
        self.entry = entry
        self.as_var = as_var

    def render(self, context):
        """Render."""
        request = context['request']
        entry = self.entry.resolve(context, True)

        plugin = entry.get_plugin(request=request)
        context[self.as_var] = plugin
        return ''


@register.tag
def get_fobi_plugin(parser, token):
    """Get the plugin.

    Note, that ``entry`` shall be a instance of
    ``fobi.models.FormElementEntry`` or ``fobi.models.FormHandlerEntry``.

    :syntax:

        {% get_fobi_plugin entry as [context_var_name] %}

    :example:

        {% get_fobi_plugin entry as plugin %}

        {% get_fobi_plugin entry as plugin %}
        {{ plugin.render }}
    """
    bits = token.contents.split()

    if 4 == len(bits):
        if 'as' != bits[-2]:
            raise TemplateSyntaxError(
                "Invalid syntax for {0}. Incorrect number of "
                "arguments.".format(
                    bits[0]
                )
            )
        as_var = bits[-1]
    else:
        raise TemplateSyntaxError(
            "Invalid syntax for {0}. See docs for valid "
            "syntax.".format(bits[0])
        )

    entry = parser.compile_filter(bits[1])

    return GetFobiPluginNode(entry=entry, as_var=as_var)

# *****************************************************************************
# ********************** Form handler plugin specific *************************
# *****************************************************************************


class GetFobiFormHandlerPluginCustomActionsNode(Node):
    """Node for ``get_fobi_form_handler_plugin_custom_actions`` tag."""

    def __init__(self, plugin, form_entry, as_var=None):
        """Constructor."""
        self.plugin = plugin
        self.form_entry = form_entry
        self.as_var = as_var

    def render(self, context):
        """Render."""
        request = context['request']
        plugin = self.plugin.resolve(context, True)
        form_entry = self.form_entry.resolve(context, True)

        context[self.as_var] = plugin.get_custom_actions(form_entry, request)
        return ''


@register.tag
def get_fobi_form_handler_plugin_custom_actions(parser, token):
    """Get the form handler plugin custom actions.

    Note, that ``plugin`` shall be a instance of
    ``fobi.models.FormHandlerEntry``.

    :syntax:

        {% get_fobi_form_handler_plugin_custom_actions
            [plugin] [form_entry] as [context_var_name] %}

    :example:

        {% get_fobi_form_handler_plugin_custom_actions
            plugin form_entry as form_handler_plugin_custom_actions %}
    """
    bits = token.contents.split()

    if 5 == len(bits):
        if 'as' != bits[-2]:
            raise TemplateSyntaxError(
                "Invalid syntax for {0}. Incorrect number of "
                "arguments.".format(
                    bits[0]
                )
            )
        as_var = bits[-1]
    else:
        raise TemplateSyntaxError(
            "Invalid syntax for {0}. See docs for valid "
            "syntax.".format(bits[0])
        )

    plugin = parser.compile_filter(bits[1])
    form_entry = parser.compile_filter(bits[2])

    return GetFobiFormHandlerPluginCustomActionsNode(
        plugin=plugin, form_entry=form_entry, as_var=as_var
    )

# *****************************************************************************
# ******************* Form wizard handler plugin specific *********************
# *****************************************************************************


class GetFobiFormWizardHandlerPluginCustomActionsNode(Node):
    """Node for ``get_fobi_form_wizard_handler_plugin_custom_actions`` tag."""

    def __init__(self, plugin, form_wizard_entry, as_var=None):
        """Constructor."""
        self.plugin = plugin
        self.form_wizard_entry = form_wizard_entry
        self.as_var = as_var

    def render(self, context):
        """Render."""
        request = context['request']
        plugin = self.plugin.resolve(context, True)
        form_wizard_entry = self.form_wizard_entry.resolve(context, True)

        context[self.as_var] = plugin.get_custom_actions(form_wizard_entry,
                                                         request)
        return ''


@register.tag
def get_fobi_form_wizard_handler_plugin_custom_actions(parser, token):
    """Get the form wizard handler plugin custom actions.

    Note, that ``plugin`` shall be a instance of
    ``fobi.models.FormWizardHandlerEntry``.

    :syntax:

        {% get_fobi_form_wizard_handler_plugin_custom_actions
            [plugin] [form_wizard_entry] as [context_var_name] %}

    :example:

        {% get_fobi_form_wizard_handler_plugin_custom_actions
            plugin form_wizard_entry as
                form_wizard_handler_plugin_custom_actions %}
    """
    bits = token.contents.split()

    if 5 == len(bits):
        if 'as' != bits[-2]:
            raise TemplateSyntaxError(
                "Invalid syntax for {0}. Incorrect number of "
                "arguments.".format(
                    bits[0]
                )
            )
        as_var = bits[-1]
    else:
        raise TemplateSyntaxError(
            "Invalid syntax for {0}. See docs for valid "
            "syntax.".format(bits[0])
        )

    plugin = parser.compile_filter(bits[1])
    form_wizard_entry = parser.compile_filter(bits[2])

    return GetFobiFormWizardHandlerPluginCustomActionsNode(
        plugin=plugin, form_wizard_entry=form_wizard_entry, as_var=as_var
    )

# *****************************************************************************
# *****************************************************************************
# *****************************************************************************
# **************************** Additional Fobi tags ***************************
# *****************************************************************************
# *****************************************************************************
# *****************************************************************************


def render_auth_link(context):
    """Render auth link."""
    if not DISPLAY_AUTH_LINK:
        return {}

    request = context.get('request', None)
    if request and request.user.is_authenticated():
        try:
            auth_url = settings.LOGOUT_URL
            auth_icon_class = 'icon-signout'
            auth_link_text = _('Log out')
        except Exception as e:
            auth_url = ''
            auth_icon_class = ''
            auth_link_text = ''
    else:
        try:
            auth_url = settings.LOGIN_URL
            auth_icon_class = 'icon-signin'
            auth_link_text = _('Log in')
        except Exception as e:
            auth_url = ''
            auth_icon_class = ''
            auth_link_text = ''

    return {
        'auth_link': auth_url,
        'auth_icon_class': auth_icon_class,
        'auth_link_text': auth_link_text
    }


register.inclusion_tag(
    'fobi/snippets/render_auth_link.html', takes_context=True
)(render_auth_link)


@register.inclusion_tag(theme.forms_list_template, takes_context=True)
def render_fobi_forms_list(context, queryset, *args, **kwargs):
    """Render the list of fobi forms.

    :syntax:

        {% render_fobi_forms_list [queryset] [show_edit_link] \
                                             [show_delete_link] \
                                             [show_export_link] %}

    :example:

        {% render_fobi_forms_list queryset show_edit_link=True \
                                           show_delete_link=False \
                                           show_export_link=False %}
    """
    request = context.get('request', None)
    show_edit_link = kwargs.get('edit_link', False)
    show_delete_link = kwargs.get('delete_link', False)
    show_export_link = kwargs.get('export_link', False)
    return {
        'show_custom_actions': (
            show_edit_link or show_delete_link or show_export_link
        ),
        'show_edit_link': show_edit_link,
        'show_delete_link': show_delete_link,
        'show_export_link': show_export_link,
    }


# *****************************************************************************
# *****************************************************************************
# *****************************************************************************
# **************************** Permission tags ********************************
# *****************************************************************************
# *****************************************************************************
# *****************************************************************************


class HasEditFormEntryPermissionsNode(Node):
    """Node for ``has_edit_form_entry_permissions`` tag."""

    def __init__(self, as_var=None):
        """Constructor."""
        self.as_var = as_var

    def render(self, context):
        """Render."""
        try:
            perms = context['perms']
        except Exception as err:
            if self.as_var:
                context[self.as_var] = False
                return ''
            else:
                return False

        perms_required = [
            'fobi.add_formentry', 'fobi.change_formentry',
            'fobi.delete_formentry',
            'fobi.add_formelemententry', 'fobi.change_formelemententry',
            'fobi.delete_formelemententry',
            'fobi.add_formhandlerentry', 'fobi.change_formhandlerentry',
            'fobi.delete_formhandlerentry',
        ]

        for perm in perms_required:
            if perm in perms:
                if self.as_var:
                    context[self.as_var] = True
                    return ''
                else:
                    return True

        if self.as_var:
            context[self.as_var] = False
            return ''
        else:
            return False


@register.tag
def has_edit_form_entry_permissions(parser, token):
    """Checks the permissions

    :syntax:

        {% has_edit_form_entry_permissions as [var_name] %}

    :example:

        {% has_edit_form_entry_permissions %}

        or

        {% has_edit_form_entry_permissions as has_permissions %}
    """
    bits = token.contents.split()

    if len(bits) not in (1, 3):
        raise TemplateSyntaxError(
            "Invalid syntax for {0}. Incorrect number of arguments.".format(
                bits[0]
            )
        )

    if 3 == len(bits):
        as_var = bits[-1]

    else:
        as_var = None

    return HasEditFormEntryPermissionsNode(as_var=as_var)

# *****************************************************************************
# *****************************************************************************
# *****************************************************************************
# **************************** General Django tags ****************************
# *****************************************************************************
# *****************************************************************************
# *****************************************************************************


class FormFieldType(object):
    """Form field type container."""

    is_checkbox = False
    is_password = False
    is_hidden = False
    is_text = False
    is_radio = False
    is_textarea = False

    def __init__(self, properties=[]):
        """Constructor.

        By default all of them are false. Provide only property
        names that should be set to True.
        """
        for property in properties:
            setattr(self, property, True)


class GetFormFieldTypeNode(Node):
    """Node for ``get_form_field_type`` tag."""

    def __init__(self, field, as_var=None):
        """Constructor."""
        self.field = field
        self.as_var = as_var

    def render(self, context):
        """Render."""
        field = self.field.resolve(context, True)
        properties = []

        if isinstance(field.field.widget, forms.CheckboxInput):
            properties.append('is_checkbox')

        if isinstance(field.field.widget, forms.CheckboxSelectMultiple):
            properties.append('is_checkbox_multiple')

        if isinstance(field.field.widget, forms.RadioSelect):
            properties.append('is_radio')

        res = FormFieldType(properties)

        context[self.as_var] = res
        return ''


@register.tag
def get_form_field_type(parser, token):
    """Get form field type.

    Syntax::

        {% get_form_field_type [field] as [context_var_name] %}

    Example::

        {% get_form_field_type form.field as form_field_type %}
        {% if form_field_type.is_checkbox %}
            ...
        {% endif %}
    """
    bits = token.contents.split()

    if 4 == len(bits):
        if 'as' != bits[-2]:
            raise TemplateSyntaxError(
                "Invalid syntax for {0}. Incorrect number of "
                "arguments.".format(
                    bits[0]
                )
            )
        as_var = bits[-1]
    else:
        raise TemplateSyntaxError(
            "Invalid syntax for {0}. See docs for valid "
            "syntax.".format(bits[0])
        )

    field = parser.compile_filter(bits[1])

    return GetFormFieldTypeNode(field=field, as_var=as_var)


class GetFormHiddenFieldsErrorsNode(Node):
    """Node for ``get_form_hidden_fields_errors`` tag."""

    def __init__(self, form, as_var=None):
        """Constructor."""
        self.form = form
        self.as_var = as_var

    def render(self, context):
        """Render."""
        form = self.form.resolve(context, True)

        hidden_fields_errors = ErrorDict()

        for field in form.hidden_fields():
            if field.errors:
                hidden_fields_errors.update({field.name: field.errors})

        context[self.as_var] = hidden_fields_errors
        return ''


@register.tag
def get_form_hidden_fields_errors(parser, token):
    """Get form hidden fields errors.

    :syntax:

        {% get_form_hidden_fields_errors [form] as [context_var_name] %}

    :example:

        {% get_form_hidden_fields_errors form as form_hidden_fields_errors %}
        {{ form_hidden_fields_errors.as_ul }}
    """
    bits = token.contents.split()

    if 4 == len(bits):
        if 'as' != bits[-2]:
            raise TemplateSyntaxError(
                "Invalid syntax for {0}. Incorrect number of "
                "arguments.".format(
                    bits[0]
                )
            )
        as_var = bits[-1]
    else:
        raise TemplateSyntaxError(
            "Invalid syntax for {0}. See docs for valid "
            "syntax.".format(bits[0])
        )

    form = parser.compile_filter(bits[1])

    return GetFormHiddenFieldsErrorsNode(form=form, as_var=as_var)



import unittest

from django.test import TestCase, RequestFactory

from fobi.base import (
    get_registered_form_element_plugins, get_registered_form_handler_plugins,
    get_registered_themes, get_registered_form_callbacks
)
from fobi.models import FormEntry
from fobi.forms import FormEntryForm
from fobi.tests.constants import TEST_FORM_NAME, TEST_FORM_SLUG
from fobi.tests.base import print_info
from fobi.tests.helpers import setup_fobi, get_or_create_admin_user

__title__ = 'fobi.tests.test_core'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2014-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('FobiCoreTest',)


class FobiCoreTest(TestCase):
    """Tests of django-fobi core functionality."""

    def setUp(self):
        """Set up."""
        setup_fobi(fobi_sync_plugins=True)

    @print_info
    def test_01_get_registered_form_element_plugins(self):
        """Test registered form element plugins.

        Calling the `get_registered_form_element_plugins`.
        """
        res = get_registered_form_element_plugins()
        self.assertTrue(len(res) > 0)
        return res

    @print_info
    def test_02_get_registered_form_handler_plugins(self):
        """Test registered form handlers.

        Calling `get_registered_form_handler_plugins`.
        """
        res = get_registered_form_handler_plugins()
        self.assertTrue(len(res) > 0)
        return res

    @print_info
    def test_03_get_registered_form_callbacks(self):
        """Test registered form callbacks.

        Calling `get_registered_form_callbacks`.
        """
        res = get_registered_form_callbacks()
        self.assertTrue(len(res) > 0)
        return res

    @print_info
    def test_04_get_registered_themes(self):
        """Test registered themes.

        Calling `get_registered_themes`.
        """
        res = get_registered_themes()
        self.assertTrue(len(res) > 0)
        return res

    def _test_form_action_url(self, form_entry, action_url):
        """Test form action URL."""
        request_factory = RequestFactory()
        request = request_factory.post(
            '/en/fobi/forms/edit/27/',
            data={
                'name': "John Doe",
                'is_public': False,
                'success_page_title': '',
                'success_page_message': '',
                'action': action_url,
            }
        )
        request.META['SERVER_NAME'] = 'localhost'
        form = FormEntryForm(request.POST, request=request,
                             instance=form_entry)

        saved = False
        try:

            if form.is_valid():
                form.save()
                saved = True
        except Exception as err:
            pass

        return saved

    def _create_form_entry(self):
        """Create form entry."""
        user = get_or_create_admin_user()
        self.assertTrue(user is not None)

        form_entry = FormEntry(
            name=TEST_FORM_NAME,
            slug=TEST_FORM_SLUG,
            user=user
        )
        form_entry.save()
        return form_entry

    @print_info
    def test_05_action_url(self):
        """Test `action` field of the URL."""
        form_entry = self._create_form_entry()

        # Local URL, OK test
        saved = self._test_form_action_url(
            form_entry, '/en/fobi/forms/edit/27/'
        )
        self.assertTrue(saved)

        # Local URL, fail test
        saved = self._test_form_action_url(
            form_entry, '/en/idontexist/'
        )
        self.assertTrue(not saved)

        # External URL, OK test
        saved = self._test_form_action_url(
            form_entry, 'http://delusionalinsanity.com/portfolio/'
        )
        self.assertTrue(saved)

        # External URL, fail test
        saved = self._test_form_action_url(
            form_entry, 'http://delusionalinsanity.com2/portfolio/'
        )
        self.assertTrue(not saved)

        # External URL, fail test
        saved = self._test_form_action_url(
            form_entry, 'http://delusionalinsanity2.com/portfolio/'
        )
        self.assertTrue(not saved)


if __name__ == '__main__':
    unittest.main()



from django.forms.fields import ChoiceField
from django.forms.widgets import Select
from django.utils.translation import ugettext_lazy as _

from fobi.base import FormFieldPlugin, form_element_plugin_registry, get_theme
from fobi.constants import (
    SUBMIT_VALUE_AS_VAL, SUBMIT_VALUE_AS_REPR
)
from fobi.helpers import get_select_field_choices, safe_text

from . import UID
from .forms import SelectInputForm
from .settings import SUBMIT_VALUE_AS

__title__ = 'fobi.contrib.plugins.form_elements.fields.' \
            'select.fobi_form_elements'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2014-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('SelectInputPlugin',)

theme = get_theme(request=None, as_instance=True)


class SelectInputPlugin(FormFieldPlugin):
    """Select field plugin."""

    uid = UID
    name = _("Select")
    group = _("Fields")
    form = SelectInputForm

    def get_form_field_instances(self, request=None):
        """Get form field instances."""
        choices = get_select_field_choices(self.data.choices)

        kwargs = {
            'label': self.data.label,
            'help_text': self.data.help_text,
            'initial': self.data.initial,
            'required': self.data.required,
            'choices': choices,
            'widget': Select(attrs={'class': theme.form_element_html_class}),
        }

        return [(self.data.name, ChoiceField, kwargs)]

    def submit_plugin_form_data(self, form_entry, request, form):
        """Submit plugin form data/process.

        :param fobi.models.FormEntry form_entry: Instance of
            ``fobi.models.FormEntry``.
        :param django.http.HttpRequest request:
        :param django.forms.Form form:
        """
        # In case if we should submit value as is, we don't return anything.
        # In other cases, we proceed further.
        if SUBMIT_VALUE_AS != SUBMIT_VALUE_AS_VAL:
            # Get the object
            value = form.cleaned_data.get(self.data.name, None)

            # Get choices
            choices = dict(get_select_field_choices(self.data.choices))

            if value in choices:
                # Handle the submitted form value

                label = safe_text(choices.get(value))

                # Should be returned as repr
                if SUBMIT_VALUE_AS == SUBMIT_VALUE_AS_REPR:
                    value = label
                # Should be returned as mix
                else:
                    value = "{0} ({1})".format(label, value)

                # Overwrite ``cleaned_data`` of the ``form`` with object
                # qualifier.
                form.cleaned_data[self.data.name] = value

                # It's critically important to return the ``form`` with
                # updated ``cleaned_data``
                return form


form_element_plugin_registry.register(SelectInputPlugin)



import os
import sys

from distutils.version import LooseVersion
from setuptools import setup, find_packages

# ****************************************************
DJANGO_INSTALLED = False
try:
    import django
    DJANGO_INSTALLED = True

    LOOSE_DJANGO_VERSION = LooseVersion(django.get_version())
    LOOSE_DJANGO_MINOR_VERSION = LooseVersion(
        '.'.join([str(i) for i in LOOSE_DJANGO_VERSION.version[0:2]])
    )

    # Loose versions
    LOOSE_VERSIONS = (
        '1.4', '1.5', '1.6', '1.7', '1.8', '1.9', '1.10', '1.11', '2.0', '2.1',
        '2.2', '3.0'
    )

    for v in LOOSE_VERSIONS:
        var_name = 'LOOSE_VERSION_{0}'.format(v.replace('.', '_'))
        globals()[var_name] = LooseVersion(v)

    # Exact versions
    EXACT_VERSIONS = LOOSE_VERSIONS[:-1]

    for i, v in enumerate(EXACT_VERSIONS):
        l_cur = globals()['LOOSE_VERSION_{0}' \
                          ''.format(LOOSE_VERSIONS[i].replace('.', '_'))]
        l_nxt = globals()['LOOSE_VERSION_{0}' \
                          ''.format(LOOSE_VERSIONS[i + 1].replace('.', '_'))]
        var_name = 'DJANGO_{0}'.format(v.replace('.', '_'))
        globals()[var_name] = (l_cur <= LOOSE_DJANGO_VERSION < l_nxt)

    # LTE list
    LTE_VERSIONS = LOOSE_VERSIONS[:-1]

    for i, v in enumerate(EXACT_VERSIONS):
        l_cur = globals()['LOOSE_VERSION_{0}' \
                          ''.format(LOOSE_VERSIONS[i].replace('.', '_'))]
        var_name = 'DJANGO_LTE_{0}'.format(v.replace('.', '_'))
        globals()[var_name] = (LOOSE_DJANGO_MINOR_VERSION <= l_cur)

    # GTE list
    GTE_VERSIONS = LOOSE_VERSIONS[:-1]

    for i, v in enumerate(EXACT_VERSIONS):
        l_cur = globals()['LOOSE_VERSION_{0}' \
                          ''.format(LOOSE_VERSIONS[i].replace('.', '_'))]
        var_name = 'DJANGO_GTE_{0}'.format(v.replace('.', '_'))
        globals()[var_name] = (
            LOOSE_DJANGO_MINOR_VERSION >= l_cur
        )

except Exception as err:
    pass

# ****************************************************
try:
    readme = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()
    readme = readme.replace('.. code-block:: none', '.. code-block::')
    screenshots = open(
        os.path.join(os.path.dirname(__file__), 'SCREENSHOTS.rst')
    ).read()
    screenshots = screenshots.replace(
        '.. image:: _static',
        '.. figure:: https://github.com/barseghyanartur/django-fobi/raw/'
        'master/docs/_static'
    )
    screenshots = screenshots.replace(
        '.. code-block:: none',
        '.. code-block::'
    )
except:
    readme = ''
    screenshots = ''

template_dirs = [
    # Core templates
    "src/fobi/templates/fobi",

    # Bootstrap 3
    "src/fobi/contrib/themes/bootstrap3/templates/bootstrap3",

    # Foundation 5
    "src/fobi/contrib/themes/foundation5/templates/foundation5",

    # DB Store widget for Foundation 5
    "src/fobi/contrib/themes/foundation5/widgets/form_handlers/"
    "db_store_foundation5_widget",

    # Simple
    "src/fobi/contrib/themes/simple/templates/simple",

    # djangocms_admin_style_theme
    "src/fobi/contrib/themes/djangocms_admin_style_theme/templates/"
    "djangocms_admin_style_theme",

    # DjangoCMS integration
    "src/fobi/contrib/apps/djangocms_integration/templates/"
    "djangocms_integration",

    # FeinCMS integration
    # "src/fobi/contrib/apps/feincms_integration/templates/"
    # "feincms_integration",

    # Mezzanine integration
    "src/fobi/contrib/apps/mezzanine_integration/templates/"
    "mezzanine_integration",

    # Content image
    "src/fobi/contrib/plugins/form_elements/content/content_image/"
    "templates/content_image",

    # DB Store
    "src/fobi/contrib/plugins/form_handlers/db_store/templates/db_store",

    # Mail
    "src/fobi/contrib/plugins/form_handlers/mail/templates/mail",

    # Http re-post
    "src/fobi/contrib/plugins/form_handlers/http_repost/templates/"
    "http_repost",

    # MailChimp importer
    "src/fobi/contrib/plugins/form_importers/mailchimp_importer/templates/"
    "mailchimp_importer",
]

static_dirs = [
    # Core static
    "src/fobi/static",

    # Bootstrap3
    "src/fobi/contrib/themes/bootstrap3/static",

    # Bootstrap3 datetime widget
    "src/fobi/contrib/themes/bootstrap3/widgets/form_elements/"
    "datetime_bootstrap3_widget/static",

    # Bootstrap3 date widget
    "src/fobi/contrib/themes/bootstrap3/widgets/form_elements/"
    "date_bootstrap3_widget/static",

    # Bootstrap3 slider widget
    "src/fobi/contrib/themes/bootstrap3/widgets/form_elements/"
    "slider_bootstrap3_widget/static",

    # Foundation5
    "src/fobi/contrib/themes/foundation5/static",

    # Foundation5 datetime widget
    "src/fobi/contrib/themes/foundation5/widgets/form_elements/"
    "datetime_foundation5_widget/static",

    # Foundation5 date widget
    "src/fobi/contrib/themes/foundation5/widgets/form_elements/"
    "date_foundation5_widget/static",

    # Simple
    "src/fobi/contrib/themes/simple/static",

    # djangocms_admin_style_theme
    "src/fobi/contrib/themes/djangocms_admin_style_theme/static",

    # DB Store
    "src/fobi/contrib/plugins/form_handlers/db_store/static",

    # Dummy
    "src/fobi/contrib/plugins/form_elements/test/dummy/static",
]

locale_dirs = [
    "src/fobi/locale/nl",
    "src/fobi/locale/ru",
    "src/fobi/locale/de",
]

templates = []
static_files = []
locale_files = []

for template_dir in template_dirs:
    templates += [os.path.join(template_dir, f)
                  for f
                  in os.listdir(template_dir)]

for static_dir in static_dirs:
    static_files += [os.path.join(static_dir, f)
                     for f
                     in os.listdir(static_dir)]

for locale_dir in locale_dirs:
    locale_files += [os.path.join(locale_dir, f)
                     for f
                     in os.listdir(locale_dir)]

version = '0.8.6'

install_requires = []
# If certain version of Django is already installed, choose version agnostic
# dependencies.
if DJANGO_INSTALLED:
    if DJANGO_1_5 or DJANGO_1_6 or DJANGO_1_7:
        install_requires = [
            'django-autoslug==1.7.1',
            # 'django-formtools>=1.0',
            'django-nine>=0.1.10',
            'django-nonefield>=0.1',
            'ordereddict>=1.1',
            'Pillow>=2.0.0',
            'requests>=1.0.0',
            'six>=1.9',
            'Unidecode>=0.04.1',
            'vishap>=0.1.5,<2.0',
        ]
    elif DJANGO_1_8:
        install_requires = [
            'django-autoslug==1.7.1',
            'django-formtools>=1.0',
            'django-nine>=0.1.10',
            'django-nonefield>=0.1',
            'ordereddict>=1.1',
            'Pillow>=2.0.0',
            'requests>=1.0.0',
            'six>=1.9',
            'Unidecode>=0.04.1',
            'vishap>=0.1.5,<2.0',
        ]
    elif DJANGO_1_9:
        install_requires = [
            'django-autoslug==1.9.3',
            'django-formtools>=1.0',
            'django-nine>=0.1.10',
            'django-nonefield>=0.1',
            'ordereddict>=1.1',
            'Pillow>=2.0.0',
            'requests>=1.0.0',
            'six>=1.9',
            'Unidecode>=0.04.1',
            'vishap>=0.1.5,<2.0',
        ]
    elif DJANGO_1_10:
        install_requires = [
            'django-autoslug==1.9.3',
            'django-formtools>=1.0',
            'django-nine>=0.1.10',
            'django-nonefield>=0.1',
            'ordereddict>=1.1',
            'Pillow>=2.0.0',
            'requests>=1.0.0',
            'six>=1.9',
            'Unidecode>=0.04.1',
            'vishap>=0.1.5,<2.0',
        ]
# Fall back to the latest dependencies
if not install_requires:
    install_requires = [
        'django-autoslug>=1.9.3',
        'django-formtools>=1.0',
        'django-nine>=0.1.10',
        'django-nonefield>=0.1',
        'ordereddict>=1.1',
        'Pillow>=2.0.0',
        'requests>=1.0.0',
        'six>=1.9',
        'Unidecode>=0.04.1',
        'vishap>=0.1.5,<2.0',
    ]

# There are also conditional PY3/PY2 requirements. Scroll down to see them.

tests_require = [
    'selenium',
]

try:
    PY2 = sys.version_info[0] == 2
    LTE_PY26 = PY2 and (7 > sys.version_info[1])
    PY3 = sys.version_info[0] == 3
    if PY3:
        install_requires.append('simplejson>=3.0.0')  # When using Python 3
        install_requires.append('easy-thumbnails>=2.3')
    else:
        install_requires.append('simplejson>=2.1.0')  # When using Python 2.*
        install_requires.append('easy-thumbnails>=1.4')
except Exception as err:
    pass

setup(
    name='django-fobi',
    version=version,
    description=("Form generator/builder application for Django done right: "
                 "customisable, modular, user- and developer- friendly."),
    long_description="{0}{1}".format(readme, screenshots),
    classifiers=[
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Environment :: Web Environment",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "License :: OSI Approved :: GNU Lesser General Public License v2 or "
        "later (LGPLv2+)",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Development Status :: 4 - Beta",
    ],
    keywords="django, form generator, form builder, visual form designer, "
             "user generated forms",
    author='Artur Barseghyan',
    author_email='artur.barseghyan@gmail.com',
    url='https://github.com/barseghyanartur/django-fobi/',
    package_dir={'': 'src'},
    packages=find_packages(where='./src'),
    license='GPL 2.0/LGPL 2.1',
    install_requires=install_requires,
    tests_require=tests_require,
    package_data={
        'fobi': templates + static_files + locale_files
    },
    include_package_data=True,
)



from django.conf import settings

from nine.user import User

__title__ = 'fobi.compat'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'AUTH_USER_MODEL',
    'User',
)

AUTH_USER_MODEL = settings.AUTH_USER_MODEL



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import autoslug.fields


class Migration(migrations.Migration):

    dependencies = [
        ('fobi', '0002_auto_20150912_1744'),
    ]

    operations = [
        migrations.AlterField(
            model_name='formelement',
            name='plugin_uid',
            field=models.CharField(verbose_name='Plugin UID', unique=True, max_length=255, editable=False, choices=[(b'regex', b'Regex'), (b'boolean', b'Boolean'), (b'select_multiple', b'Select multiple'), (b'checkbox_select_multiple', b'Checkbox select multiple'), (b'textarea', b'Textarea'), (b'select_model_object', b'Select model object'), (b'datetime', b'DateTime'), (b'text', b'Text'), (b'decimal', b'Decimal'), (b'float', b'Float'), (b'email', b'Email'), (b'date_drop_down', b'Date drop down'), (b'radio', b'Radio'), (b'null_boolean', b'Null boolean'), (b'time', b'Time'), (b'date', b'Date'), (b'integer', b'Integer'), (b'hidden', b'Hidden'), (b'password', b'Password'), (b'slug', b'Slug'), (b'select', b'Select')]),
        ),
        migrations.AlterField(
            model_name='formelemententry',
            name='plugin_uid',
            field=models.CharField(max_length=255, verbose_name='Plugin name', choices=[(b'regex', b'Regex'), (b'boolean', b'Boolean'), (b'select_multiple', b'Select multiple'), (b'checkbox_select_multiple', b'Checkbox select multiple'), (b'textarea', b'Textarea'), (b'select_model_object', b'Select model object'), (b'datetime', b'DateTime'), (b'text', b'Text'), (b'decimal', b'Decimal'), (b'float', b'Float'), (b'email', b'Email'), (b'date_drop_down', b'Date drop down'), (b'radio', b'Radio'), (b'null_boolean', b'Null boolean'), (b'time', b'Time'), (b'date', b'Date'), (b'integer', b'Integer'), (b'hidden', b'Hidden'), (b'password', b'Password'), (b'slug', b'Slug'), (b'select', b'Select')]),
        ),
        migrations.AlterField(
            model_name='formentry',
            name='slug',
            field=autoslug.fields.AutoSlugField(editable=False, populate_from=b'name', unique=True, verbose_name='Slug'),
        ),
        migrations.AlterField(
            model_name='formhandler',
            name='plugin_uid',
            field=models.CharField(verbose_name='Plugin UID', unique=True, max_length=255, editable=False),
        ),
        migrations.AlterField(
            model_name='formhandlerentry',
            name='plugin_uid',
            field=models.CharField(max_length=255, verbose_name='Plugin name'),
        ),
        migrations.AlterField(
            model_name='formwizardentry',
            name='slug',
            field=autoslug.fields.AutoSlugField(editable=False, populate_from=b'name', unique=True, verbose_name='Slug'),
        ),
    ]
